#ifndef ENCODER_H
#define	ENCODER_H

#include <stdint.h>
#include "esp_err.h"

#define CONFIG_ENCODER_THREAD_STACK_SIZE 8192
#define CONFIG_ENCODER_THREAD_PRIORITY 20
#define CONFIG_ENCODER_CPU 0 

typedef void (*encoder_callback_t)(int, int8_t, uint32_t);

typedef struct {
	int8_t A;            		 ///< A pin
	int8_t B;            		 ///< B pin
	uint8_t state;      		 ///< Current state's machine state
	int32_t counter;    		 ///< Current counter value
	encoder_callback_t callback; ///< Callback function
	int callback_id;             ///< Callback id
	uint8_t deferred;            ///< Deferred callback?
} encoder_h_t;

typedef struct {
	encoder_h_t *h;
	int8_t dir;
	uint32_t counter;
} encoder_deferred_data_t;

esp_err_t encoder_setup(int8_t a, int8_t b, encoder_h_t **h);
esp_err_t encoder_unsetup(encoder_h_t *h);
esp_err_t encoder_read(encoder_h_t *h, int32_t *val);
esp_err_t encoder_write(encoder_h_t *h, int32_t val);
esp_err_t encoder_register_callback(encoder_h_t *h, encoder_callback_t callback, int id, uint8_t deferred);

#endif	/* ENCODER_H */